@@ -34,6 +34,7 @@ from s3transfer.utils import calculate_range_parameter
 from s3transfer.utils import FunctionContainer
 from s3transfer.utils import CountCallbackInvoker
 from s3transfer.utils import StreamReaderProgress
+from s3transfer.utils import DeferredOpenFile
 from s3transfer.tasks import Task
 from s3transfer.tasks import SubmissionTask
 
@@ -106,14 +107,33 @@ class DownloadOutputManager(object):
         """
         self._transfer_coordinator.submit(
             self._io_executor,
-            IOWriteTask(
-                self._transfer_coordinator,
-                main_kwargs={
-                    'fileobj': fileobj,
-                    'data': data,
-                    'offset': offset,
-                }
-            )
+            self.get_io_write_task(fileobj, data, offset)
+         )
+
+    def get_io_write_task(self, fileobj, data, offset):
+        """Get an IO write task for the requested set of data
+
+        This task can be ran immediately or be submitted to the IO executor
+        for it to run.
+
+        :type fileobj: file-like object
+        :param fileobj: The file-like object to write to
+
+        :type data: bytes
+        :param data: The data to write out
+
+        :type offset: integer
+        :param offset: The offset to write the data to in the file-like object
+
+        :returns: An IO task to be used to write data to a file-like object
+        """
+        return IOWriteTask(
+            self._transfer_coordinator,
+            main_kwargs={
+                'fileobj': fileobj,
+                'data': data,
+                'offset': offset,
+            }
         )
 
     def get_final_io_task(self):
@@ -171,7 +191,8 @@ class DownloadFilenameOutputManager(DownloadOutputManager):
         return f
 
     def _get_fileobj(self, filename):
-        f = self._osutil.open(filename, 'wb')
+        f = DeferredOpenFile(
+            filename, mode='wb', open_function=self._osutil.open)
         # Make sure the file gets closed and we remove the temporary file
         # if anything goes wrong during the process.
         self._transfer_coordinator.add_failure_cleanup(f.close)
@@ -244,16 +265,18 @@ class DownloadNonSeekableOutputManager(DownloadOutputManager):
                 data = write['data']
                 logger.debug("Queueing IO offset %s for fileobj: %s",
                              write['offset'], fileobj)
-                self._transfer_coordinator.submit(
-                    self._io_executor,
-                    IOStreamingWriteTask(
-                        self._transfer_coordinator,
-                        main_kwargs={
-                            'fileobj': fileobj,
-                            'data': data,
-                        }
-                    )
-                )
+                super(
+                    DownloadNonSeekableOutputManager, self).queue_file_io_task(
+                        fileobj, data, offset)
+
+    def get_io_write_task(self, fileobj, data, offset):
+        return IOStreamingWriteTask(
+            self._transfer_coordinator,
+            main_kwargs={
+                'fileobj': fileobj,
+                'data': data,
+            }
+        )
 
 
 class DownloadSubmissionTask(SubmissionTask):
@@ -353,18 +376,13 @@ class DownloadSubmissionTask(SubmissionTask):
         # Get any associated tags for the get object task.
         get_object_tag = download_output_manager.get_download_task_tag()
 
-        # Callback invoker to submit the final io task once the download
-        # is complete.
-        finalize_download_invoker = CountCallbackInvoker(
-            self._get_final_io_task_submission_callback(
-                download_output_manager, io_executor
-            )
-        )
-        finalize_download_invoker.increment()
+        # Get the final io task to run once the download is complete.
+        final_task = download_output_manager.get_final_io_task()
+
         # Submit the task to download the object.
         self._transfer_coordinator.submit(
             request_executor,
-            GetObjectTask(
+            ImmediatelyWriteIOGetObjectTask(
                 transfer_coordinator=self._transfer_coordinator,
                 main_kwargs={
                     'client': client,
@@ -377,13 +395,11 @@ class DownloadSubmissionTask(SubmissionTask):
                     'download_output_manager': download_output_manager,
                     'io_chunksize': config.io_chunksize,
                 },
-                done_callbacks=[finalize_download_invoker.decrement]
+                done_callbacks=[final_task]
             ),
             tag=get_object_tag
         )
 
-        finalize_download_invoker.finalize()
-
     def _submit_ranged_download_request(self, client, config, osutil,
                                         request_executor, io_executor,
                                         download_output_manager,
@@ -492,15 +508,16 @@ class GetObjectTask(Task):
                     response['Body'], callbacks)
 
                 current_index = start_index
-                chunks = iter(
-                    lambda: streaming_body.read(io_chunksize), b'')
+                chunks = DownloadChunkIterator(streaming_body, io_chunksize)
                 for chunk in chunks:
                     # If the transfer is done because of a cancellation
                     # or error somewhere else, stop trying to submit more
                     # data to be written and break out of the download.
                     if not self._transfer_coordinator.done():
-                        download_output_manager.queue_file_io_task(
-                            fileobj, chunk, current_index)
+                        self._handle_io(
+                            download_output_manager, fileobj, chunk,
+                            current_index
+                        )
                         current_index += len(chunk)
                     else:
                         return
@@ -518,6 +535,21 @@ class GetObjectTask(Task):
                 continue
         raise RetriesExceededError(last_exception)
 
+    def _handle_io(self, download_output_manager, fileobj, chunk, index):
+        download_output_manager.queue_file_io_task(fileobj, chunk, index)
+
+
+class ImmediatelyWriteIOGetObjectTask(GetObjectTask):
+    """GetObjectTask that immediately writes to the provided file object
+
+    This is useful for downloads where it is known only one thread is
+    downloading the object so there is no reason to go through the
+    overhead of using an IO queue and executor.
+    """
+    def _handle_io(self, download_output_manager, fileobj, chunk, index):
+        task = download_output_manager.get_io_write_task(fileobj, chunk, index)
+        task()
+
 
 class IOWriteTask(Task):
     def _main(self, fileobj, data, offset):
@@ -581,6 +613,36 @@ class CompleteDownloadNOOPTask(Task):
         pass
 
 
+class DownloadChunkIterator(object):
+    def __init__(self, body, chunksize):
+        """Iterator to chunk out a downloaded S3 stream
+
+        :param body: A readable file-like object
+        :param chunksize: The amount to read each time
+        """
+        self._body = body
+        self._chunksize = chunksize
+        self._num_reads = 0
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        chunk = self._body.read(self._chunksize)
+        self._num_reads += 1
+        if chunk:
+            return chunk
+        elif self._num_reads == 1:
+            # Even though the response may have not had any
+            # content, we still want to account for an empty object's
+            # existance so return the empty chunk for that initial
+            # read.
+            return chunk
+        raise StopIteration()
+
+    next = __next__
+
+
 class DeferQueue(object):
     """IO queue that defers write requests until they are queued sequentially.
 
